#!/usr/bin/env python
# -*- coding: utf-8 -*-

from launchpadlib.launchpad import Launchpad
import os
import sys
import logging
import ConfigParser
import traceback


def auth(config, logging):
    '''Function for autorization on Launchpad'''

    try:
        launchpad = Launchpad.login_with(
            config['appname'],
            config['env'],
            credentials_file=config['credfile'],
            launchpadlib_dir=config['cachedir']
        )
    except Exception as e:
        logging.error(
            'Unexpected error was returned when connected to launchpad'
        )
        logging.error('Error: {}'.format(e))
        logging.error(traceback.print_exc(e))
        sys.exit(1)

    return launchpad


def confparser():
    '''Function for parsing config file'''

    config_file = '/etc/lpbugmanage/lpbugmanage.conf'

    logging.basicConfig(
        level=logging.INFO,
        format=u'[%(asctime)s] --%(levelname)s-- %(message)s',
    )

    if os.path.exists(config_file):
        config = ConfigParser.ConfigParser()
        config.read(config_file)
        result = {}

        for i in config.options('bugmanage'):
            param = config.get('bugmanage', i)

            if i == 'status':
                result[i] = [j.strip() for j in param.split(',')]

            elif i == 'credfile':
                if (not os.path.exists(param) or
                   os.stat(param).st_size == 0):
                    logging.error(
                        'Error with credentials store file in {}'.format(param)
                    )
                    sys.exit(1)

                else:
                    result[i] = config.get('bugmanage', i)

            else:
                result[i] = config.get('bugmanage', i)

        return result

    else:
        logging.error("Couldn't find config file at '{}'".format(config_file))
        sys.exit(1)


def main():
    config = confparser()
    msg = {
        'info_sr': '{} was targeted to series {}',
        'info_ml': '{} was targeted to milestone {}',
        'warn': '{} has already been targeted to series {}',
        'error': 'Unexpected error was returned when targeted {}',
        'error_lp': 'A fix for this bug has already been requested for {}',
    }

    if len(sys.argv) > 1 and sys.argv[1] == 'test':
        logging.basicConfig(
            level=logging.INFO,
            format=u'[%(asctime)s] --%(levelname)s-- %(message)s',
        )
    else:
        logging.basicConfig(
            level=logging.INFO,
            format=u'[%(asctime)s] --%(levelname)s-- %(message)s',
            filename=config['logfile']
        )

    launchpad = auth(config, logging)
    bugs = launchpad.bugs
    project = launchpad.projects
    fuel = project[config['distr']]

    # Creating a list of current series and their milestones
    ser_mile = {}

    for ser in fuel.series:
        ser_mile[ser.name] = [
            mile.name
            for mile in ser.all_milestones if mile.is_active
        ]
        if len(ser_mile[ser.name]) > 0:
            ser_mile[ser.name] = min(ser_mile[ser.name])

    # Current development series
    dev_series = fuel.development_focus.name

    # Create date of current development series
    date_created = str(fuel.development_focus.date_created).split()[0]

    # Current development milestones
    active_mile = fuel.development_focus.active_milestones
    cur_dev_ml = [mlstn.name for mlstn in active_mile]
    min_mile = min(cur_dev_ml)

    # Series of project
    series_list = [srs.name for srs in fuel.series if srs.name != 'future']

    bug_list = fuel.searchTasks(
        created_since=date_created,
        status=config['status']
    )

    #For dry-run mode
    if len(sys.argv) > 1 and sys.argv[1] == 'test':
        bug_count = 30
        bug_list = bug_list[:bug_count]

    for bug in bug_list:

        bug_link = bug.bug.web_link

        if bug.milestone:
            bug_milestone = bug.milestone.series_target.name

            if dev_series != bug_milestone:
                if bug_milestone != 'future':
                    target_s = series_list[series_list.index(bug_milestone):-1]

                if len(sys.argv) > 1 and sys.argv[1] == 'test':
                    for trgt in target_s:
                        logging.info(msg['info_sr'].format(bug_link, trgt))

                    for i in bugs[bug.bug.id].bug_tasks:
                        ml_link = '{}/{}'.format(
                            config['milestone'],
                            ser_mile[trgt]
                        )

                else:
                    for trgt in target_s:
                        try:
                            bug_id = bug.bug.id
                            target = '{}/{}'.format(config['series'], trgt)
                            task_link = bugs[bug_id].addTask(target=target)

                            if not task_link.milestone:
                                ml_link = '{}/{}'.format(
                                    config['milestone'],
                                    ser_mile[trgt]
                                )
                                task_link.milestone = ml_link
                                task_link.lp_save()
                            else:
                                ml_link = '{}/{}'.format(
                                    config['milestone'],
                                    ser_mile[dev_series]
                                )
                                task_link.milestone = ml_link
                                task_link.lp_save()
                            logging.info(
                                msg['info_sr'].format(bug_link, trgt)
                            )
                        except Exception as e:
                            if e.content == msg['error_lp'].format(trgt):
                                logging.warn(
                                    msg['warn'].format(bug_link, trgt)
                                )
                                logging.warn(traceback.print_exc(e))
                            else:
                                logging.error(msg['error'].format(bug_link))
                                logging.error('Error: {}'.format(e))
                                logging.error(traceback.print_exc(e))

        else:
            ml_link = '{}/{}'.format(config['milestone'], min_mile)
            bug.milestone = ml_link

            if len(sys.argv) > 1 and sys.argv[1] == 'test':
                logging.info(msg['info_ml'].format(bug_link, min_mile))
            else:
                try:
                    bug.lp_save()
                    logging.info(msg['info_ml'].format(bug_link, min_mile))
                except Exception as e:
                    logging.error(msg['error'].format(bug_link))
                    logging.error('Error: {}'.format(e))
                    logging.error(traceback.print_exc(e))

if __name__ == "__main__":
    main()
